"""Node Visitor for integer overflow sites."""

from pycparser import c_ast

class IntOverflowVisitor(c_ast.NodeVisitor):

  """pyparser NodeVisitor for integer overflow sites."""

  def __init__(self):
    """Constructor method with a list to keep the nodes that matches the site
    type.
    """
    self.nodes = []

  def visit_UnaryOp(self, node):
    """Rules for site matching."""
    # Fix: also check for post-increment (e.g. i++, op p++)
    if(node.op =='++' or node.op =='p++'):
      self.nodes.append(node.expr)

    for c_name, c in node.children():
      self.visit(c)

  def visit_BinaryOp(self, node):
    """Rules for site matching."""
    if(node.op == '+' or node.op == '*'):
      self.nodes.append(node)

    for c_name, c in node.children():
      self.visit(c)

  def visit_Assignment(self, node):
    """Rules for site matching."""
    if(node.op == '+=' or node.op == '*='):
      self.nodes.append(node.lvalue)

    for c_name, c in node.children():
      self.visit(c)

