"""Node Visitor for variable access sites."""

from pycparser import c_ast

class VariableAccessVisitor(c_ast.NodeVisitor):

  """pyparser NodeVisitor for variable access sites."""

  def __init__(self):
    """Constructor method with a list to keep the nodes that matches the site
    type.
    """
    self.nodes = []

  def visit_ArrayRef(self, node):
    """Rules for site matching."""
    self.nodes.append(node.name)
    if(isinstance(node.subscript, c_ast.ID)):
      self.nodes.append(node.subscript)

    elif(isinstance(node.subscript, c_ast.StructRef)):
      self.nodes.append(node.subscript.name)

    for c_name, c in node.children():
      self.visit(c)

  #only base types?
  def visit_ExprList(self, node):
    """Rules for site matching."""
    for c_name, c in node.children():
      if(isinstance(c, c_ast.ID) and not c.name.isupper()):
        self.nodes.append(c)

      elif(isinstance(c, c_ast.StructRef)):
        self.nodes.append(c.name)

      self.visit(c)

  def visit_Assignment(self, node):
    """Rules for site matching."""
    if(isinstance(node.rvalue, c_ast.ID) and not node.rvalue.name.isupper()):
      self.nodes.append(node.rvalue)

    elif(isinstance(node.rvalue, c_ast.StructRef)):
      self.nodes.append(node.rvalue.name)

    if(node.op != '='):
      if (isinstance(node.lvalue, c_ast.ID)):
        self.nodes.append(node.lvalue)

      elif(isinstance(node.lvalue, c_ast.StructRef)):
        self.nodes.append(node.lvalue.name)

    for c_name, c in node.children():
      self.visit(c)

  def visit_BinaryOp(self, node):
    """Rules for site matching."""
    for c_name, c in node.children():
      if(isinstance(c, c_ast.ID) and not c.name.isupper()):
        self.nodes.append(c)

      elif(isinstance(c, c_ast.StructRef)):
        self.nodes.append(c.name)

      self.visit(c)

  def visit_UnaryOp(self, node):
    """Rules for site matching."""
    if(node.op != '&' and node.op != 'sizeof'):
      if(isinstance(node.expr, c_ast.ID)):
        self.nodes.append(node.expr)
      elif(isinstance(node.expr, c_ast.StructRef)):
        self.nodes.append(node.expr.name)

    for c_name, c in node.children():
      self.visit(c)

  def visit_Return(self, node):
    """Rules for site matching."""
    if(isinstance(node.expr, c_ast.ID) and not node.expr.name.isupper()):
      self.nodes.append(node.expr)

    elif(isinstance(node.expr, c_ast.StructRef)):
      self.nodes.append(node.expr.name)

    for c_name, c in node.children():
      self.visit(c)

  def visit_Decl(self, node):
    """Rules for site matching."""
    if(isinstance(node.init, c_ast.ID) and not node.init.name.isupper()):
      self.nodes.append(node.init)

    elif(isinstance(node.init, c_ast.StructRef)):
      self.nodes.append(node.init.name)

    for c_name, c in node.children():
      self.visit(c)

  def visit_Cast(self, node):
    """Rules for site matching."""
    if(isinstance(node.expr, c_ast.ID) and not node.expr.name.isupper()):
      self.nodes.append(node.expr)

    elif(isinstance(node.expr, c_ast.StructRef)):
      self.nodes.append(node.expr.name)

    for c_name, c in node.children():
      self.visit(c)

  def visit_If(self, node):
    """Rules for site matching."""
    for c_name, c in node.children():
      if(isinstance(c, c_ast.ID) and not c.name.isupper()):
        self.nodes.append(c)

      elif(isinstance(c, c_ast.StructRef)):
        self.nodes.append(c.name)

      self.visit(c)

  def visit_For(self, node):
    """Rules for site matching."""
    for c_name, c in node.children():
      if(isinstance(c, c_ast.ID) and not c.name.isupper()):
        self.nodes.append(c)

      elif(isinstance(c, c_ast.StructRef)):
        self.nodes.append(c.name)

      self.visit(c)

  def visit_While(self, node):
    """Rules for site matching."""
    for c_name, c in node.children():
      if(isinstance(c, c_ast.ID) and not c.name.isupper()):
        self.nodes.append(c)

      elif(isinstance(c, c_ast.StructRef)):
        self.nodes.append(c.name)

      self.visit(c)

  def visit_DoWhile(self, node):
    """Rules for site matching."""
    for c_name, c in node.children():
      if(isinstance(c, c_ast.ID) and not c.name.isupper()):
        self.nodes.append(c)

      elif(isinstance(c, c_ast.StructRef)):
        self.nodes.append(c.name)

      self.visit(c)

  def visit_Switch(self, node):
    """Rules for site matching."""
    for c_name, c in node.children():
      if(isinstance(c, c_ast.ID) and not c.name.isupper()):
        self.nodes.append(c)

      elif(isinstance(c, c_ast.StructRef)):
        self.nodes.append(c.name)

      self.visit(c)

  def visit_TernaryOp(self, node):
    """Rules for site matching."""
    for c_name, c in node.children():
      if(isinstance(c, c_ast.ID) and not c.name.isupper()):
        self.nodes.append(c)

      elif(isinstance(c, c_ast.StructRef)):
        self.nodes.append(c.name)

      self.visit(c)

  def visit_StructRef(self, node):
    """Rules for site matching."""
    if(node.type == '->'):
      self.nodes.append(node.name)

    for c_name, c in node.children():
      self.visit(c)

