import unittest
from ocse import site
from ocse import extractor
import os

class TestExtractor(unittest.TestCase):
  def setUp(self):
    self.extractor = extractor.Extractor("ocse/test/samples/common")

  def test_extractor_constructor(self):
    self.assertIsInstance(self.extractor, extractor.Extractor)

  def test_set_file_list(self):
    self.assertEqual(3, len(self.extractor.files))

  def test_variable_access_sites(self):
    sites = self.extractor.variable_access_sites()
    self.assertIsInstance(sites, list)
    for my_site in sites:
      self.assertIsInstance(my_site, site.Site)

  def test_variable_access_sites_type(self):
    sites = self.extractor.variable_access_sites()
    for site in sites:
      self.assertEqual("variable", site.site_type)

  def test_variable_access_sites_access(self):
    variable_access_extractor = extractor.Extractor("ocse/test/samples/common")
    variable_access_sites = variable_access_extractor.variable_access_sites()
    self.assertEqual(50, len(variable_access_sites))

  def test_div_by_zero_sites(self):
    sites = self.extractor.div_by_zero_sites()
    self.assertIsInstance(sites, list)
    for my_site in sites:
      self.assertIsInstance(my_site, site.Site)

  def test_div_by_zero_sites_type(self):
    sites = self.extractor.div_by_zero_sites()
    for site in sites:
      self.assertEqual("div_by_zero", site.site_type)

  def test_div_by_zero_sites_division_by_zero(self):
    div_by_zero_extractor = extractor.Extractor("ocse/test/samples/common")
    div_by_zero_sites = div_by_zero_extractor.div_by_zero_sites()
    self.assertEqual(3, len(div_by_zero_sites))
    self.assertEqual(109, div_by_zero_sites[0].line)
    self.assertEqual("int result = (int)(100.0 / data);", div_by_zero_sites[0].code)
    self.assertEqual(128, div_by_zero_sites[1].line)
    self.assertEqual("int result = (int)(100.0 / data);", div_by_zero_sites[1].code)
    self.assertEqual(198, div_by_zero_sites[2].line)
    self.assertEqual("int result = (int)(100.0 / data);", div_by_zero_sites[2].code)

  def test_null_ptr_deref_sites(self):
    sites = self.extractor.null_ptr_deref_sites()
    self.assertIsInstance(sites, list)
    for my_site in sites:
      self.assertIsInstance(my_site, site.Site)

  def test_null_ptr_deref_sites_type(self):
    sites = self.extractor.null_ptr_deref_sites()
    for site in sites:
      self.assertEqual("null_ptr_deref", site.site_type)

  def test_null_ptr_deref_sites_sites(self):
    null_ptr_deref_extractor = extractor.Extractor("ocse/test/samples/null_ptr_deref")
    null_ptr_deref_sites = null_ptr_deref_extractor.null_ptr_deref_sites()
    self.assertEqual(9, len(null_ptr_deref_sites))

  def test_int_overflow_sites(self):
    sites = self.extractor.int_overflow_sites()
    self.assertIsInstance(sites, list)
    for my_site in sites:
      self.assertIsInstance(my_site, site.Site)

  def test_int_overflow_sites_type(self):
    sites = self.extractor.int_overflow_sites()
    for site in sites:
      self.assertEqual("int_overflow", site.site_type)

  def test_int_overflow_sites_sites(self):
    int_overflow_extractor = extractor.Extractor("ocse/test/samples/int_overflow")
    int_overflow_sites = int_overflow_extractor.int_overflow_sites()
    self.assertEqual(9, len(int_overflow_sites))

  def test_int_underflow_sites(self):
    sites = self.extractor.int_underflow_sites()
    self.assertIsInstance(sites, list)
    for my_site in sites:
      self.assertIsInstance(my_site, site.Site)

  def test_int_underflow_sites_type(self):
    sites = self.extractor.int_underflow_sites()
    for site in sites:
      self.assertEqual("int_underflow", site.site_type)

  def test_int_underflow_sites_sites(self):
    int_underflow_extractor = extractor.Extractor("ocse/test/samples/int_overflow")
    int_underflow_sites = int_underflow_extractor.int_underflow_sites()
    self.assertEqual(9, len(int_underflow_sites))

  def test_buffer_write_sites(self):
    sites = self.extractor.buffer_write_sites()
    self.assertIsInstance(sites, list)
    for my_site in sites:
      self.assertIsInstance(my_site, site.Site)

  def test_buffer_write_sites_type(self):
    sites = self.extractor.buffer_write_sites()
    for site in sites:
      self.assertEqual("buffer_write", site.site_type)

  def test_buffer_write_sites_sites(self):
    buffer_write_extractor = extractor.Extractor("ocse/test/samples/buffer_write")
    buffer_write_sites = buffer_write_extractor.buffer_write_sites()
    self.assertEqual(31, len(buffer_write_sites))

  def test_buffer_read_sites(self):
    sites = self.extractor.buffer_read_sites()
    self.assertIsInstance(sites, list)
    for my_site in sites:
      self.assertIsInstance(my_site, site.Site)

  def test_buffer_read_sites_type(self):
    sites = self.extractor.buffer_read_sites()
    for site in sites:
      self.assertEqual("buffer_read", site.site_type)

  def test_buffer_read_sites_sites(self):
    buffer_read_extractor = extractor.Extractor("ocse/test/samples/buffer_read")
    buffer_read_sites = buffer_read_extractor.buffer_read_sites()
    self.assertEqual(10, len(buffer_read_sites))

  def test_write_what_where_sites(self):
    sites = self.extractor.write_what_where_sites()
    self.assertIsInstance(sites, list)
    for my_site in sites:
      self.assertIsInstance(my_site, site.Site)

  def test_write_what_where_sites_type(self):
    sites = self.extractor.write_what_where_sites()
    for site in sites:
      self.assertEqual("write_what_where", site.site_type)

  def test_write_what_where_sites_sites(self):
    write_what_where_extractor = extractor.Extractor("ocse/test/samples/write_what_where")
    write_what_where_sites = write_what_where_extractor.write_what_where_sites()
    self.assertEqual(8, len(write_what_where_sites))

  def test_return_sites(self):
    sites = self.extractor.return_sites()
    self.assertIsInstance(sites, list)
    for my_site in sites:
      self.assertIsInstance(my_site, site.Site)

  def test_return_sites_type(self):
    sites = self.extractor.return_sites()
    for site in sites:
      self.assertEqual("return", site.site_type)

  def test_return_sites_sites(self):
    return_extractor = extractor.Extractor("ocse/test/samples/return")
    return_sites = return_extractor.return_sites()
    self.assertEqual(4, len(return_sites))

  def test_to_xml(self):
    div_by_zero_extractor = extractor.Extractor("ocse/test/samples/common")
    div_by_zero_sites = div_by_zero_extractor.div_by_zero_sites()
    extractor.Extractor.to_xml(div_by_zero_sites)
    f = open("sites_list.xml", "r")
    line = 'init'
    lines = []
    while(line != ""):
      line = f.readline()
      lines.append(line)

    f.close()
    self.assertEqual("<?xml version='1.0' encoding='utf-8'?>\n", lines[0])
    self.assertEqual('<sites>\n', lines[1])
    self.assertEqual('  <file name="sample_divide_by_zero.c">\n', lines[2])
    self.assertEqual('    <site type="div_by_zero" line="109" info="data" val="buggy">\n', lines[3])
    self.assertEqual('      <code><![CDATA[int result = (int)(100.0 / data);]]></code>\n', lines[4])
    self.assertEqual('    </site>\n', lines[5])
    self.assertEqual('    <site type="div_by_zero" line="128" info="data" val="good">\n', lines[6])
    self.assertEqual('      <code><![CDATA[int result = (int)(100.0 / data);]]></code>\n', lines[7])
    self.assertEqual('    </site>\n', lines[8])
    self.assertEqual('    <site type="div_by_zero" line="198" info="data" val="good">\n', lines[9])
    self.assertEqual('      <code><![CDATA[int result = (int)(100.0 / data);]]></code>\n', lines[10])
    self.assertEqual('    </site>\n', lines[11])
    self.assertEqual('  </file>\n', lines[12])
    self.assertEqual('</sites>\n', lines[13])
    self.assertEqual(15, len(lines))
    os.remove('sites_list.xml')

  def test_to_xml_empty_sites_list(self):
    div_by_zero_sites = []
    extractor.Extractor.to_xml(div_by_zero_sites, 'empty.xml')
    f = open("empty.xml", "r")
    line = 'init'
    lines = []
    while(line != ""):
      line = f.readline()
      lines.append(line)

    f.close()
    self.assertEqual("<?xml version='1.0' encoding='utf-8'?>\n", lines[0])
    self.assertEqual('<sites/>\n', lines[1])
    self.assertEqual(3, len(lines))
    os.remove('empty.xml')

  def test_buggy_sites(self):
    buggy_extractor = extractor.Extractor("ocse/test/samples/common")
    buggy_sites = buggy_extractor.buggy_sites()
    self.assertEqual(4, len(buggy_sites))

  def test_set_buggy_attr_div_by_zero(self):
    buggy_extractor = extractor.Extractor("ocse/test/samples/common")
    buggy_sites = buggy_extractor.buggy_sites()
    div_by_zero_extractor = extractor.Extractor("ocse/test/samples/common")
    div_by_zero_sites = div_by_zero_extractor.div_by_zero_sites()
    extractor.Extractor.set_buggy_attr(div_by_zero_sites, buggy_sites)
    bugs = 0
    bugged = []
    for site in div_by_zero_sites:
      if(site.buggy):
        bugs += 1
        bugged.append(site)
    self.assertEqual(1, bugs)
    self.assertEqual(109, bugged[0].line)

  def test_set_buggy_attr_variable_access(self):
    buggy_extractor = extractor.Extractor("ocse/test/samples/common")
    buggy_sites = buggy_extractor.buggy_sites()
    variable_access_extractor = extractor.Extractor("ocse/test/samples/common")
    variable_access_sites = variable_access_extractor.variable_access_sites()
    extractor.Extractor.set_buggy_attr(variable_access_sites, buggy_sites)
    bugs = 0
    bugged = []
    for site in variable_access_sites:
      if(site.buggy):
        bugs += 1
        bugged.append(site)

    self.assertEqual(1, bugs)
    self.assertEqual(30, bugged[0].line)

  def test_set_buggy_attr_null_ptr_deref(self):
    buggy_extractor = extractor.Extractor("ocse/test/samples/null_ptr_deref")
    buggy_sites = buggy_extractor.buggy_sites()
    null_ptr_deref_extractor = extractor.Extractor("ocse/test/samples/null_ptr_deref")
    null_ptr_deref_sites = null_ptr_deref_extractor.null_ptr_deref_sites()
    extractor.Extractor.set_buggy_attr(null_ptr_deref_sites, buggy_sites)
    bugs = 0
    bugged = []
    for site in null_ptr_deref_sites:
      if(site.buggy):
        bugs += 1
        bugged.append(site)

    self.assertEqual(4, bugs)

  def test_set_buggy_attr_int_overflow(self):
    buggy_extractor = extractor.Extractor("ocse/test/samples/int_overflow")
    buggy_sites = buggy_extractor.buggy_sites()
    int_overflow_extractor = extractor.Extractor("ocse/test/samples/int_overflow")
    int_overflow_sites = int_overflow_extractor.int_overflow_sites()
    extractor.Extractor.set_buggy_attr(int_overflow_sites, buggy_sites)
    bugs = 0
    bugged = []
    for site in int_overflow_sites:
      if(site.buggy):
        bugs += 1
        bugged.append(site)

    self.assertEqual(2, bugs)

  def test_set_buggy_attr_int_underflow(self):
    buggy_extractor = extractor.Extractor("ocse/test/samples/int_overflow")
    buggy_sites = buggy_extractor.buggy_sites()
    int_underflow_extractor = extractor.Extractor("ocse/test/samples/int_overflow")
    int_underflow_sites = int_underflow_extractor.int_underflow_sites()
    extractor.Extractor.set_buggy_attr(int_underflow_sites, buggy_sites)
    bugs = 0
    bugged = []
    for site in int_underflow_sites:
      if(site.buggy):
        bugs += 1
        bugged.append(site)

    self.assertEqual(2, bugs)

  def test_set_buggy_attr_buffer_write(self):
    buggy_extractor = extractor.Extractor("ocse/test/samples/buffer_write")
    buggy_sites = buggy_extractor.buggy_sites()
    buffer_write_extractor = extractor.Extractor("ocse/test/samples/buffer_write")
    buffer_write_sites = buffer_write_extractor.buffer_write_sites()
    extractor.Extractor.set_buggy_attr(buffer_write_sites, buggy_sites)
    bugs = 0
    bugged = []
    for site in buffer_write_sites:
      if(site.buggy):
        bugs += 1
        bugged.append(site)

    self.assertEqual(5, bugs)

  def test_set_buggy_attr_buffer_read(self):
    buggy_extractor = extractor.Extractor("ocse/test/samples/buffer_read")
    buggy_sites = buggy_extractor.buggy_sites()
    buffer_read_extractor = extractor.Extractor("ocse/test/samples/buffer_read")
    buffer_read_sites = buffer_read_extractor.buffer_read_sites()
    extractor.Extractor.set_buggy_attr(buffer_read_sites, buggy_sites)
    bugs = 0
    bugged = []
    for site in buffer_read_sites:
      if(site.buggy):
        bugs += 1
        bugged.append(site)

    self.assertEqual(2, bugs)

  def test_set_buggy_attr_write_what_where(self):
    buggy_extractor = extractor.Extractor("ocse/test/samples/write_what_where")
    buggy_sites = buggy_extractor.buggy_sites()
    write_what_where_extractor = extractor.Extractor("ocse/test/samples/write_what_where")
    write_what_where_sites = write_what_where_extractor.write_what_where_sites()
    extractor.Extractor.set_buggy_attr(write_what_where_sites, buggy_sites)
    bugs = 0
    bugged = []
    for site in write_what_where_sites:
      if(site.buggy):
        bugs += 1
        bugged.append(site)

    self.assertEqual(4, bugs)

  def test_set_buggy_attr_return(self):
    buggy_extractor = extractor.Extractor("ocse/test/samples/return")
    buggy_sites = buggy_extractor.buggy_sites()
    return_extractor = extractor.Extractor("ocse/test/samples/return")
    return_sites = return_extractor.return_sites()
    extractor.Extractor.set_buggy_attr(return_sites, buggy_sites)
    bugs = 0
    bugged = []
    for site in return_sites:
      if(site.buggy):
        bugs += 1
        bugged.append(site)

    self.assertEqual(2, bugs)

if __name__ == '__main__':
      unittest.main()

