# Convert all selected sites, which are in a Ockham Criteria XML 
# formatted files to CSV format to do the Ockham analysis.
# The output format is:
#  file_name, line number, warningName
import xml.etree.ElementTree as ET
import argparse, os.path, sys

parser = argparse.ArgumentParser(description='Convert Ockham Criteria XML formatted file to CSV.')
parser.add_argument('-b', '--buggy', help='include buggy sites', action='store_true')
parser.add_argument('-n', '--non-buggy', help='include non-buggy sites', action='store_true')
parser.add_argument('-f', '--sites-file', help='xml sites file name', metavar='FILE', required=True)
parser.add_argument('-w', '--warn-name', help='warning name', required=True)
args = parser.parse_args()

if not os.path.exists(args.sites_file):
  print "File '%s' does not exist!" % args.sites_file
  sys.exit(1)

# if neither -b nor -n selected, print all
if args.buggy == False and args.non_buggy == False:
  args.buggy = args.non_buggy = True

tree = ET.parse(args.sites_file)
root = tree.getroot()

for child in root:
  for grand in child:
    if (grand.get('val') == 'buggy' and args.buggy) or (grand.get('val') != 'buggy' and args.non_buggy):
        print "{0}, {1}, {2}".format(child.get('name'), grand.get('line'), args.warn_name)

# end of sitesxml2csv.py
